/*
  Macro           FindAlso
  Author          Carlo Hogeveen
  Website         eCarlo.nl/tse
  Compatibility   Windows GUI TSE v4 with Courier New font,
                  Linux TSE
  Version         1.0.1   4 Feb 2025


  This tool lets you use simple letters when searching for accented letters and
  special characters.

  When you use this tool to search using a string containing ordinary letters
  and characters, the tool also finds:
  - The accented variants of the letters.
  - Ligatures (single characters that are combinations of letters).
  - Symbols described by a commonly used alternative of ordinary letters.
  - Some other non-ASCII characters.

  The tool also always uses search options "ix" in addition to the options that
  you provide. All TSE's search options are allowed and supported.

  For more details see the EXAMPLES section.


  LIMITATIONS

  The tool does not work for Console TSE.
  For GUI TSE the tool does not work with other fonts than Courier New yet.


  KNOWN RISK

  This tool has not been been extensively tested yet.


  INSTALLATION

  Just copy this file to TSE's "mac" directory, and compile it there, for
  example by opening it there in TSE and then applying the Macro Compile menu.


  USAGE

  Just execute the tool like a macro, for example:
  - By using the Macro Execute menu.
  - By adding it to TSE's Potpourri menu and starting it from there.
  - By attaching it to a key in your .ui file.


  EXAMPLES
    Common symbols:
      "1/4" also finds "".
      "1/2" also finds "".
      "3/4" also finds "".
      "(c)" also finds "".
      "(r)" also finds "".
    Ligatures:
      "AE" also finds "".
      "ae" also finds "".
      "OE" also finds "".
      "oe" also finds "".
      "sz" also finds "".
    Common symbol:
      "tm" also finds "".
    German custom:
      "ue" also finds "".
    Superscript alternatives:
      "1" also finds "".
      "2" also finds "".
      "3" also finds "".
    Accented and other single-letter alternatives:
      "a" case-insensitively also finds the letters in "".
      "c" case-insensitively also finds the letters in "".
      "e" case-insensitively also finds the letters in "".
      "i" case-insensitively also finds the letters in "".
      "l" case-insensitively also finds the letters in "".
      "m" case-insensitively also finds the letters in "".
      "n" case-insensitively also finds the letters in "".
      "o" case-insensitively also finds the letters in "".
      "s" case-insensitively also finds the letters in "".
      "u" case-insensitively also finds the letters in "".
      "y" case-insensitively also finds the letters in "".
      "x" case-insensitively also finds the letters in "".
      "z" case-insensitively also finds the letters in "".
    Any of the above as part of words:
      "Espanol"   also finds "Espaol".
      "Francais"  also finds "Franais".
      "Portugues" also finds "Portugus".


  TODO
    MUST
    SHOULD
    COULD
    - Allow other compatible fonts in GUI TSE.
    - Make "!" and "?" also find "" and "" respectively.
    - Make "," and "." also find their look-alikes.
    - Make any quoting character find any other quoting character.
    WONT


  HISTORY

  v1        3 Feb 2025
    Initial release.

  v1.0.1    4 Feb 2025
    Fixed three small bugs:
    - The tool used _EDIT_HISTORY_ instead of _FIND_HISTORY_.
    - When the search-string was not found, the tool reported that with its
      internal search-string instead of with the input search-string.
    - You now get a proper error if the input-string expands to a too-long
      search-string.
*/






// Start of compatibility restrictions and mitigations ...

#ifdef LINUX
  #define WIN32 FALSE
#endif

#ifndef WIN32
  16-bit versions of TSE are not supported. You need at least TSE v4.
#endif

#ifndef EDITOR_VERSION
  Editor Version is older than TSE v3. You need at least TSE v4.
#endif

#if EDITOR_VERSION < 4000h
  Editor Version is older than TSE v4. You need at least TSE v4.
#endif

// End of compatibility restrictions and mitigations.




integer proc is_compatible_TSE_variant()
  integer result = TRUE

  #ifndef LINUX
    result = isGUI()
  #endif

  return(result)
end is_compatible_TSE_variant


integer proc has_compatible_font_name()
  integer result = TRUE

  #ifndef LINUX
    integer font_flags                   = 0
    string  font_name     [MAXSTRINGLEN] = ''
    integer font_size                    = 0

    GetFont(font_name, font_size, font_flags)

    result = (font_name == 'Courier New')
  #endif

  return(result)
end has_compatible_font_name


proc optimize(var string s)
  integer i      = 1
  integer j      = 0

  while i <= Length(s)
    if (s[i] in ' ', 'i', 'x')
      s = s[1: i - 1] + s[i + 1: MAXSTRINGLEN]
    else
      j = i + 1

      while j <= Length(s)
        if EquiStr(s[j], s[i])
          s = s[1: j - 1] + s[j + 1: MAXSTRINGLEN]
        else
          j = j + 1
        endif
      endwhile

      i = i + 1
    endif
  endwhile

  s = 'ix' + s
end optimize


proc Main()
  integer classified                      = 0
  integer i                               = 1
  integer ok                              = TRUE
  integer old_part_length                 = 1
  string  replacement      [MAXSTRINGLEN] = ''
  string  search_options             [11] = ''
  string  search_string_in [MAXSTRINGLEN] = ''
  string  search_string    [MAXSTRINGLEN] = ''

  if     not is_compatible_TSE_variant()
    Warn('Abort: This tool does not run in the Windows Console variant of TSE.')
  elseif not has_compatible_font_name()
    Warn('Abort: Currently only font "Courier New" is permitted in Windows.')
  else
    if  Ask('Search for (including same letters with accents):',
            search_string_in, _FIND_HISTORY_)
    and Ask('Options IX[ABGLVW] (Ignore-case reg-eXp All-files Back Global Local Words):',
            search_options  , _FIND_OPTIONS_HISTORY_)
      search_string = search_string_in
      optimize(search_options)

      while ok
      and   i <= Length(search_string)
        replacement = ''

        if     search_string [i] == '\'
          i = i + 1   // Skip checking the next character.
        elseif search_string [i] == '['
          classified = classified + 1
        elseif search_string [i] == ']'
          classified = classified - 1
        elseif not classified
          old_part_length = 3

          case search_string [i: 3]
            when '1/4'        replacement = '|{1/4}'
            when '1/2'        replacement = '|{1/2}'
            when '3/4'        replacement = '|{3/4}'
            when '(C)', '(c)' replacement = '|{([Cc])}'
            when '(R)', '(r)' replacement = '|{([Rr])}'
            otherwise
              old_part_length = 2

              case search_string [i: 2]
                when 'AE', 'ae' replacement = '[]|{[Aa][Ee]}'
                when 'OE', 'oe' replacement = '[]|{[Oo][Ee]}'
                when 'sz'       replacement = '|{[Ss][Zz]}'
                when 'tm'       replacement = '|{t[Mm]}'
                when 'ue'       replacement = '|{[Uu][Ee]}'
                otherwise
                  old_part_length = 1

                  case search_string [i]
                    when '1'      replacement = '[1]'
                    when '2'      replacement = '[2]'
                    when '3'      replacement = '[3]'
                    when 'A', 'a' replacement = '[Aa]'
                    when 'C', 'c' replacement = '[Cc]'
                    when 'E', 'e' replacement = '[Ee]'
                    when 'I', 'i' replacement = '[Ii]'
                    when 'L', 'l' replacement = '[Ll]'
                    when 'M', 'm' replacement = '[Mm]'
                    when 'N', 'n' replacement = '[Nn]'
                    when 'O', 'o' replacement = '[Oo]'
                    when 'S', 's' replacement = '[Ss]'
                    when 'U', 'u' replacement = '[Uu]'
                    when 'Y', 'y' replacement = '[Yݥy]'
                    when 'X', 'x' replacement = '[Xx]'
                    when 'Z', 'z' replacement = '[Zz]'
                  endcase
              endcase
          endcase
        endif

        if Length(replacement)
          if Length(search_string) - old_part_length + Length(replacement) > MAXSTRINGLEN
            Warn('Input-string expands to a too long search-string.')
            ok = FALSE
          else
            search_string = Format(search_string[1: i - 1],
                                   replacement,
                                   search_string[i + old_part_length: MAXSTRINGLEN])
          endif
        endif

        i = i + Max(Length(replacement), 1)
      endwhile

      if      ok
      and not Find(search_string, search_options)
        Message(search_string_in; 'not found.')
      endif
    endif
  endif

  PurgeMacro(CurrMacroFilename())
end Main

